import QtQuick 2.12
import Process 0.1
import StorageMgr 1.0
import Components 1.0
import ".."

// Holds a list of active network connections
Item {
    id: root

    property bool isRefreshing: refreshTimer.running || ifconfigProcess.processState == Process.Running || ifconfigProcessWireless.processState == Process.Running || iwconfigProcess.processState == Process.Running || wpaCliProcess.processState == Process.Running

    // Wired connection information
    property bool wireDisconnected: false
    property bool wiredConfigured: false
    property bool wiredEnabled: false
    property string wiredIp
    property string wiredBroadcast
    property string wiredNetmask
    property string wiredMac
    // Wireless connection information
    property bool wirelessConfigured: false
    property bool wirelessEnabled: false
    property bool encrypted: false
    property string wirelessIp
    property string wirelessBroadcast
    property string wirelessNetmask
    property string wirelessMac
    property string signalStrengthRatio
    property string signalStrength: signalStrengthText(signalStrengthRatio) + application.emptyString
    property string currentESSID: "not connected"
    property bool debug: true

    property alias list: activeConnections

    Component.onCompleted: {
        refreshList()
    }

    onWireDisconnectedChanged: {
    }

    function debugLog(str) {
        if (root.debug) {
            console.log(str)
        }
    }

    function fetchConfigured() {
        // Find out which interfaces have files in /etc/network/interfaces.d, regardless
        // of whether or not they are currently active
        var files = storageMgr.listFiles("/etc/network/interfaces.d", "*.cfg")
        if(files.length === 0) {
            root.wiredConfigured = true
        }

        for (var i = 0; i < files.length; i++) {
            if (files[i].indexOf(main.wiredAdapter) >= 0) {
                root.wiredConfigured = true
            } else if (files[i].indexOf(main.wifiAdapter) >= 0) {
                root.wirelessConfigured = true
            }
        }
    }

    function signalStrengthText(ratio) {
        var text = ""
        if (ratio.indexOf("/") < 0) {
            text = qsTr("Unknown") + application.emptyString
        } else {
            var parts = ratio.split("/")
            var num = parseInt(parts[0], 10), den = parseInt(parts[1], 10)
            var numRatio = (1.0 * num) / den
            if (numRatio >= 0.8) {
                text = qsTr("Very Strong") + application.emptyString
            } else if (numRatio >= 0.6) {
                text = qsTr("Strong") + application.emptyString
            } else if (numRatio >= 0.4) {
                text = qsTr("Weak") + application.emptyString
            } else {
                text = qsTr("Very Weak") + application.emptyString
            }
        }

        return text
    }

    function parseString(str, name)
    {
        var mac = str.match(/ether (.+?)\s/)[1]
        if (name === main.wifiAdapter) {
            root.wirelessMac = mac
        } else if (name === main.wiredAdapter) {
            root.wiredMac = mac
        }

        var ipSearch = str.match(/inet (.+?) /)
        if (ipSearch === null) {
            return // no IP found. Abort and do not update values
        }

        var ip = ipSearch[1]
        var broadcast = str.match(/broadcast (.+?)\s/)[1]
        var mask = str.match(/netmask (.+?)\s/)[1]
        if (name === main.wifiAdapter) {
            root.wirelessEnabled = true
            root.wirelessBroadcast = broadcast
            root.wirelessNetmask = mask
            root.wirelessIp = ip
        } else if (name === main.wiredAdapter) {
            root.wiredEnabled = true
            root.wiredBroadcast = broadcast
            root.wiredNetmask = mask
            root.wiredIp = ip
        }
    }

    function resetFields() {
        activeConnections.clear()
        root.wiredConfigured = false
        root.wirelessConfigured = false
        root.wirelessEnabled = false
        root.encrypted = false
        root.wirelessBroadcast = ""
        root.wirelessNetmask = ""
        root.wirelessIp = ""
        root.wirelessMac = ""
        root.wiredEnabled = false
        root.wiredBroadcast = ""
        root.wiredNetmask = ""
        root.wiredIp = ""
        root.wiredMac = ""
        root.signalStrengthRatio = ""
    }

    function printStatus() {
        console.log("NetworkConnections: Status begin")
        console.log(" * Wire Disconnected:", root.wireDisconnected)
        if (root.wirelessEnabled) {
            console.log(" * Wifi -", "Encrypted:", root.encrypted, "IP Address:", root.wirelessIp, "Broadcast:", root.wirelessBroadcast, "Subnet Mask:", root.wirelessNetmask, "Signal Strength:", root.signalStrength, "MAC Address:", root.wirelessMac)
        } else {
            console.log(" * Wifi not enabled")
        }
        if (root.wiredEnabled) {
            console.log(" * Ethernet -", "IP Address:", root.wiredIp, "Broadcast:", root.wiredBroadcast, "Subnet Mask:", root.wiredNetmask, "MAC Address:", root.wiredMac)
        } else {
            console.log(" * Wired not enabled")
        }
        console.log("NetworkConnections: Status end")
    }

    function findInterfaceIndex(intf) {
        for (var i = 0; i < activeConnections.count; i++) {
            if (activeConnections.get(i).intf === intf) {
                return i
            }
        }
        return -1
    }

    function refreshList() {
        resetFields()
        if (wireWatcher.processState !== Process.Running) {
            wireWatcher.start()
        }

        debugLog("sending command... "+ifconfigProcess.program)
        refreshTimer.restart()

        fetchConfigured()

        if (root.wiredConfigured) {
            var wiredConn = {"intf": main.wiredAdapter, "essid": "Wired Ethernet", "encrypted": false}
            activeConnections.append(wiredConn)
        }
        if (root.wirelessConfigured) {
            var conn = {"intf": main.wifiAdapter, "essid": "", "encrypted": false}

            activeConnections.insert(0, conn)
        }
        ifconfigProcess.start()
        ifconfigProcessWireless.start()

        console.log("Active Connections:", activeConnections.count)
    }

    StorageMgr {
        id: storageMgr
    }

    Timer {
        id: refreshTimer
        interval: 1000
        repeat: false; running: false
    }

    Process{
        id: ifconfigProcess
        program: "ifconfig"
        arguments: [main.wiredAdapter]

        onReadyReadStandardOutput: {
            var output = new String(readAllStandardOutput())
            debugLog( "output: " + output)
            parseString(output, main.wiredAdapter)
        }

        onReadyReadStandardError: {
            var output = new String(readAllStandardError())
            debugLog( "ifconfig error output: " + output)
        }

    }

    Process {
        id: iwconfigProcess
        program: "iwconfig"
        arguments: [main.wifiAdapter]

        onReadyReadStandardOutput: {
            var output = new String(readAllStandardOutput())
            debugLog( "iwconfig output: " + output)
            if(output.match(/wlp/)) {
                var essidSearch = output.match(/ESSID:"(.+?)"\s/)
                root.currentESSID = essidSearch ? essidSearch[1] : "not connected"
                var strengthMatch = output.match(/Link Quality=(.+?)\s/)
                if (strengthMatch) {
                    root.signalStrengthRatio = strengthMatch[1]
                }
            }

            wpaCliProcess.start()
        }
    }

    Process {
        id: ifconfigProcessWireless
        program: "ifconfig"
        arguments: [main.wifiAdapter]

        onReadyReadStandardOutput: {
            var output = new String(readAllStandardOutput())
            debugLog( "output: " + output)
            parseString(output, main.wifiAdapter)
            iwconfigProcess.start()
        }

        onReadyReadStandardError: {
            var output = new String(readAllStandardError())
            debugLog( "ifconfig error output: " + output)
        }
    }

    Process {
        id: wpaCliProcess
        program: "sudo"
        arguments: ["wpa_cli", "status"]
        onReadyReadStandardOutput: {
            var output = new String(readAllStandardOutput())
            if (output.indexOf("key_mgmt=NONE") >= 0) {
                root.encrypted = false
            } else {
                root.encrypted = true
            }
            if (root.wirelessEnabled) {
                var index = root.findInterfaceIndex(main.wifiAdapter)

                if (index >= 0) {
                    activeConnections.setProperty(index, "essid", root.currentESSID)
                    activeConnections.setProperty(index, "encrypted", root.encrypted)
                } else {
                    debugLog("Warning: Wireless interface "+main.wifiAdapter+" not found in list")
                }
            }
        }
    }

    Process {
        id: wireWatcher
        program: "scripts/watch.sh"
        arguments: ["/sys/class/net/"+main.wiredAdapter+"/carrier"]
        onReadyReadStandardOutput: {
            var output = new String(readAllStandardOutput())
            output = output.trim()
            if (output === "1") {
                root.wireDisconnected = false
            } else if (output === "0") {
                root.wireDisconnected = true
            } else {
                console.log("wire watcher: unknown output:", output)
            }
        }
    }

    ListModel {
        id: activeConnections
        // interface is a reserved keyword in javascript
        //ListElement { intf: "wlp6s0"; encrypted: true; essid: "npwireless2.4" }
        //ListElement { intf: "wlp5s0"; encrypted: true; essid: "npwireless2.4" }
        //ListElement { intf: "enp2s0"; encrypted: false; essid: "Wired Ethernet" }
    }

}
